using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design.Serialization;
using System.Reflection;
using System.Text;
using System.Globalization;

namespace AppliedDataBindingSample {
  class PositionsConverter : ExpandableObjectConverter {

    public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType) {
      // We can convert from a Positions type to a string
      return( destinationType == typeof(string) );
    }

    public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType) {
      // We can convert from a string to a Positions type
      if( sourceType == typeof(string) ) { return true; }
      return base.CanConvertFrom(context, sourceType);
    }

    public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo info, object value) {

      // If converting from a string
      if( value is string ) {
        // Build a Positions type
        try {
          // Get Positions properties
          string propertyList = (string)value;
          string[] properties = propertyList.Split('-');
          return new Positions(int.Parse(properties[0]), int.Parse(properties[1]), int.Parse(properties[2]));
        }
        catch { }
        throw new ArgumentException("The arguments were not valid.");
      }
      return base.ConvertFrom(context, info, value);
    }

    public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType) {

      // If source value is a Positions type
      if( value is Positions ) {
        // Convert to string
        if( (destinationType == typeof(string)) ) {
          Positions Positions = (Positions)value;
          return string.Format("{0}-{1}-{2}", Positions.Firsts, Positions.Seconds, Positions.Thirds);
        }
      }

      // Base ConvertTo if neither string or InstanceDescriptor required
      return base.ConvertTo(context, culture, value, destinationType);
    }
  }
}
