using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Deployment.Application;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.IsolatedStorage;
using System.Net;
using System.Net.NetworkInformation;
using System.Security;
using System.Security.Permissions;
using System.Text;
using System.Web.Services.Protocols;
using System.Windows.Forms;

using Wahoo.WahooScores;

namespace Wahoo {
  public partial class MainForm : Form {

//TODO: Would love it if wahoo2 remember its settings

    private int score;

    public MainForm() {
      InitializeComponent();

      // HACK: Add a textbox so that we can get the arrow keys
      Controls.Add(new System.Windows.Forms.TextBox());

      // Redirect web service component URL to base site
      // from which application is being launched
      this.scoresService.Url = UrlJiggler.JiggleUrl(this.scoresService.Url);

      // Refresh form size and location from previous session
      this.Location = Properties.Settings.Default.MainFormLocation;
      this.Size = Properties.Settings.Default.MainFormSize;
      FormWindowState bob = Properties.Settings.Default.MainFormWindowState;
      // WindowState can only be set when UIPermission is All Windows and Events
      this.WindowState = Properties.Settings.Default.MainFormWindowState;
    }
    public MainForm(int columns, int rows) : this() {
      this.wahooGameControl.Columns = columns;
      this.wahooGameControl.Rows = rows;
    }

    private void newGameToolStripMenuItem_Click(object sender, EventArgs e) {

      if( this.wahooGameControl.IsPlaying ) {
        if( MessageBox.Show("Quit current game?", "Wahoo!", MessageBoxButtons.YesNo) == DialogResult.No ) {
          return;
        }
      }

      this.score = 0;
      this.wahooGameControl.New();
      this.statusToolStripStatusLabel.Text = "Playing (F3 to Pause)";
      this.scoreToolStripStatusLabel.Text = "Score: " + this.score;
    }
    private void pauseResumeGameToolStripMenuItem_Click(object sender, EventArgs e) {
      if( !this.wahooGameControl.IsPlaying ) return;

      if( this.wahooGameControl.IsPaused ) {
        this.wahooGameControl.Resume();
        this.statusToolStripStatusLabel.Text = "Playing (F3 to Pause)";
      }
      else {
        this.wahooGameControl.Pause();
        this.statusToolStripStatusLabel.Text = "Paused (F3 to Resume)";
      }
    }
    private void getHighScoresToolStripMenuItem_Click(object sender, EventArgs e) {
      this.GetHighScores();
    }
    private void viewCachedHighScoresToolStripMenuItem_Click(object sender, EventArgs e) {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.DefaultExt = ".txt";
      dlg.Filter = "Text Files (*.txt)|*.txt|All files (*.*)|*.*";

      if( dlg.ShowDialog() == DialogResult.OK ) {
        using( Stream stream = dlg.OpenFile() )
        using( StreamReader reader = new StreamReader(stream) ) {
          MessageBox.Show(reader.ReadToEnd(), "Wahoo! Cached High Scores");
        }
      }
    }
    private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
      this.Close();
    }
    private void keysToolStripMenuItem_Click(object sender, EventArgs e) {
      string s = "The Wahoo! control keys are:\r\n\r\n" +
        "Left:\tj and Left Arrow\r\n" +
        "Right:\tl and Right Arrow\r\n" +
        "Rotate:\ti and Up Arrow\r\n" +
        "Drop:\tk and Down Arrow";
      MessageBox.Show(s, "Wahoo! Keys");
    }
    private void aboutWahooToolStripMenuItem_Click(object sender, EventArgs e) {
      AboutBox aboutBox = new AboutBox();
      aboutBox.ShowDialog();
    }

    private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {

      // Save form size and location for next session
      switch( this.WindowState ) {
        case FormWindowState.Maximized:
          Properties.Settings.Default.MainFormLocation = this.RestoreBounds.Location;
          Properties.Settings.Default.MainFormSize = this.RestoreBounds.Size;
          Properties.Settings.Default.MainFormWindowState = FormWindowState.Maximized;
          break;
        case FormWindowState.Minimized:
          Properties.Settings.Default.MainFormLocation = this.RestoreBounds.Location;
          Properties.Settings.Default.MainFormSize = this.RestoreBounds.Size;
          Properties.Settings.Default.MainFormWindowState = FormWindowState.Normal;
          break;
        case FormWindowState.Normal:
          Properties.Settings.Default.MainFormLocation = this.Location;
          Properties.Settings.Default.MainFormSize = this.Size;
          Properties.Settings.Default.MainFormWindowState = FormWindowState.Normal;
          break;
      }

      Properties.Settings.Default.Save();
    }
    private void MainForm_KeyDown(object sender, KeyEventArgs e) {
      //Trace.WriteLine(string.Format("MainForm_KeyDown({0})", e.KeyData));
      e.Handled = this.wahooGameControl.ProcessKey(e.KeyData);
      e.SuppressKeyPress = e.Handled;
    }
    private void MainForm_Deactivate(object sender, EventArgs e) {
      if( !this.wahooGameControl.IsPlaying || this.wahooGameControl.IsPaused ) return;
      this.wahooGameControl.Pause();
      this.statusToolStripStatusLabel.Text = "Paused (F3 to Resume)";
    }

    private void wahooGameControl_GameOver() {
      this.statusToolStripStatusLabel.Text = "Game Over (F2 to Play)";

      // Let the player know and ask them if they'd like to
      // send their high score to the server
      DialogResult dr = MessageBox.Show("Game Over...\r\n\r\nSend score of " + this.score + " to Wahoo! high score server?", "Wahoo!", MessageBoxButtons.YesNo);
      if( dr == DialogResult.Yes ) {
        this.SendHighScore(this.score);
      }
    }
    private void wahooGameControl_LinesRemoved(int linesRemoved) {
      // Normalize the speed
      int speed = (this.wahooGameControl.Speed < 10 ? 1 : this.wahooGameControl.Speed);

      // A Wahoo! is double
      if( linesRemoved == 4 ) {
        this.score += linesRemoved * speed * 2;
        this.scoreToolStripStatusLabel.Text = "Score: " + this.score + " (Wahoo!)";
      }
      else {
        this.score += linesRemoved * speed;
        this.scoreToolStripStatusLabel.Text = "Score: " + this.score;
      }
    }

    // Check permission
    static bool HavePermission(IPermission perm) {
      try { perm.Demand(); }
      catch( SecurityException ) { return false; }
      return true;
    }

    #region Wahoo Web Service

    private void GetHighScores() {

      WahooScore[] scores;

      try {
        // Get high scores
        //ScoresService service = (ScoresService)GetServiceForAppBase(typeof(ScoresService));
        scores = this.scoresService.GetScores();
      }
      catch( Exception e ) {
        MessageBox.Show("Unable to get high scores\r\n" + e.Message + "\r\n" + this.scoresService.Url, "Wahoo!");
        return;
      }

      // Show scores
      StringBuilder sb = new StringBuilder();
      for( int i = 0; i != Math.Min(scores.Length, 10); ++i ) {
        sb.AppendFormat("{0}\t{1}\r\n", scores[i].Score, scores[i].Name);
      }

      DialogResult dr = MessageBox.Show(sb.ToString() + "\r\nCache these scores locally?", "Wahoo! Top 10 Scores", MessageBoxButtons.YesNo);

      // Cache scores safely
      if( dr == DialogResult.Yes ) {
        // Check for permissions to do this
        // (By default, won't have this permission in the Internet Zone)
        if( !HavePermission(new FileDialogPermission(FileDialogPermissionAccess.Save)) ) {
          string s = "This application does not have permission to save files on this machine.";
          MessageBox.Show(s, "Wahoo!");
          return;
        }

        SaveFileDialog dlg = new SaveFileDialog();
        dlg.DefaultExt = ".txt";
        dlg.Filter = "Text Files (*.txt)|*.txt|All files (*.*)|*.*";

        // NOTE: Not allowed unless we have FileIOPermission : )
        dlg.AddExtension = true;
        dlg.FileName = "Wahoo! High Scores.txt";

        if( dlg.ShowDialog() == DialogResult.OK ) {
          // NOTE: Not allowed to call dlg.FileName
          using( Stream stream = dlg.OpenFile() )
          using( StreamWriter writer = new StreamWriter(stream) ) {
            writer.Write(sb.ToString());
          }
        }
      }
    }
    private void SendHighScore(int score) {

      HighScoreWhoForm dlg = new HighScoreWhoForm();
      dlg.Score = score;
      dlg.PlayerName = "Anon";

      if( dlg.ShowDialog() != DialogResult.OK ) return;

      try {
        // Send high score
        //ScoresService service = (ScoresService)GetServiceForAppBase(typeof(ScoresService));
        this.scoresService.RegisterScore(dlg.PlayerName, score);
        MessageBox.Show("Your high score was registered on the Wahoo! server", "Wahoo!");
      }
      catch( Exception e ) {
        MessageBox.Show("Unable to send high score: " + e.Message, "Wahoo!");
        return;
      }
    }

    #endregion
  }
}