﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

#endregion

namespace DragAndDropSample {
  partial class MainForm : Form {

    // KeyState Values (not available in WinForms)
    [FlagsAttribute]
    enum KeyState {
      LeftMouse = 1,
      RightMouse = 2,
      ShiftKey = 4,
      CtrlKey = 8,
      MiddleMouse = 16,
      AltKey = 32,
    }

    public MainForm() {
      InitializeComponent();
    }

    private void DragAndDropSampleForm_Load(object sender, EventArgs e) {
      // Enable drop target
      this.dropTargetTextBox.AllowDrop = true;
    }

    private void dragSourceLabel_MouseDown(object sender, MouseEventArgs e) {

//      this.DoDragDrop(this.dragSourceLabel.Text, DragDropEffects.Copy);

      // Start a drag and drop operation
      DragDropEffects supportedEffects = DragDropEffects.Copy | DragDropEffects.Move;
      string dragData = this.dragSourceLabel.Text;
      //  DragDropEffects dragEffect = DoDragDrop(dragData, supportedEffects);
      DragDropEffects dragEffect = this.dragSourceLabel.DoDragDrop(dragData, supportedEffects);

      // If the effect was move, remove the text of the button
      // If the effect was a copy, we don’t have anything to do
      if( dragEffect == DragDropEffects.Move ) {
        this.dragSourceLabel.Text = "";
      }
      this.Cursor = Cursors.Default;
    }

    private void dropTargetTextBox_DragEnter(object sender, DragEventArgs e) {
//      // Could check against DataFormats.Text as well
//      if( e.Data.GetDataPresent(typeof(string)) ) {
//        e.Effect = DragDropEffects.Copy;
//      }
//      else {
//        e.Effect = DragDropEffects.None;
//      }
      SetDropEffect(e);
    }

    private void dropTargetTextBox_DragOver(object sender, DragEventArgs e) {
      SetDropEffect(e);
    }

    private void dropTargetTextBox_DragDrop(object sender, DragEventArgs e) {
      // Handles both DragDropEffect.Move and DragDropEffect.Copy
      
      // Retrieve drag data
      string dragData = (string)e.Data.GetData(typeof(string));
      // Drop data onto target
      this.dropTargetTextBox.Text = dragData;
    }

    private void SetDropEffect(DragEventArgs e) {

      KeyState keyState = (KeyState)e.KeyState;

      // If the data is a string, we can handle it
      if( e.Data.GetDataPresent(typeof(string)) ) {
        // If only Ctrl is pressed, copy it
        if( (keyState & KeyState.CtrlKey) == KeyState.CtrlKey) {
          e.Effect = DragDropEffects.Copy;
        }
        else { // Else, move it
          e.Effect = DragDropEffects.Move;
        }
      }
      // We don't like the data, so do not allow anything
      // e.Effect = DragDropEffects.None by default
    }

    private void dragSourceLabel_GiveFeedback(object sender, GiveFeedbackEventArgs e) {
      // Use this to override the default drag/drop UI behavior
      // and provide custom UI behavior ie custom cursors and whatever else

      // Is a drag and drop move in effect?
      if( e.Effect == DragDropEffects.Move ) {
        // Use the custom move cursor
        e.UseDefaultCursors = false;
        this.Cursor = new Cursor(new MemoryStream(Properties.Resources.CustomMove)); 
        return;
      }
      
      // Is a drag and drop copy in effect?
      if( e.Effect == DragDropEffects.Copy) {
        // Use the custom copy cursor
        e.UseDefaultCursors = false;
        this.Cursor = new Cursor(new MemoryStream(Properties.Resources.CustomCopy));
        return;
      }

      // Use default cursors for any other drag and drop effect
      // e.UseDefaultCursors = true by default
    }

    private void dragSourceLabel_QueryContinueDrag(object sender, QueryContinueDragEventArgs e) {
      // Get the current cursor position, relative to the form
      Point cursorPosition = this.PointToClient(Cursor.Position);
      // Cancel the drag and drop operation if the cursor
      // is dragged outside the form
      if( !this.ClientRectangle.Contains(cursorPosition)) {
        e.Action = DragAction.Cancel;
      }
    }
  }
}